@@ -2,12 +2,13 @@
 #include <iostream>
 #include "Framework.hpp"
 #include "Interface.hpp"
-#include "Buffer.hpp"
 #include <math.h>
+#include "SDL_image.h"
+
 
 Framework::Framework(void *surf, const Configuration& conf, Networkstate initial)
- : field(this), output(this), surface((SDL_Surface*)surf), version(conf.version),
-   paused(1), timeunit(0), lasttime(SDL_GetTicks()), frames(0), state(initial), xdiff(0), camera(conf.width, conf.height)
+ : field(this), output(this), surface((SDL_Surface*)surf),
+   paused(1), timeunit(7), lasttime(SDL_GetTicks()), frames(0), state(initial), xdiff(0), camera(conf.width, conf.height)
 {
 	/* initialize OpenGL */
 	resetGL();
@@ -18,7 +19,7 @@ Framework::Framework(void *surf, const Configuration& conf, Networkstate initial
 	if (conf.fullscreen) SDL_WM_ToggleFullScreen(surface);
 
 	/* set unpaused state (grab input) */
-	togglePause(false, false);
+	togglePause(false, true);
 }
 
 Framework::~Framework()
@@ -63,45 +64,30 @@ void Framework::loop()
 				done = true;
 				break;
 			case SDL_USEREVENT:
-				switch (event.user.code)
+				if (((TimerData*)event.user.data1)->timer == NULL)
 				{
-				case NET2_EXTERNAL:
-					if (((TimerData*)event.user.data1)->timer == NULL)
-					{
-						/* this means our timer has gone inactive and we are pleased to stop our work! */
-					} else {
-						((TimerData*)event.user.data1)->receiver->action(((TimerData*)event.user.data1)->event);
-					}
-					break;
-				case NET2_ERROREVENT:
-					printf("Error: %s(%d)\n", NET2_GetEventError(&event), NET2_GetSocket(&event));
-					break;
-				case NET2_UDPRECEIVEEVENT:
-					UDPpacket *p = NULL;
-					p = NET2_UDPRead(NET2_GetSocket(&event));
-					while (p != NULL) // if we get NULL we are done
-					{
-						Peer* peer = getCreatePeer(p->address.host);
-						receivePacket(peer, (char*)p->data, p->len);
-						NET2_UDPFreePacket(p);
-						p = NET2_UDPRead(NET2_GetSocket(&event));
-					}
-					break;
+					/* this means our timer has gone inactive and we are pleased to stop our work! */
+				} else {
+					((TimerData*)event.user.data1)->receiver->action(((TimerData*)event.user.data1)->event);
 				}
+				break;
 			}
 		}
-
 		int tdiff = SDL_GetTicks() - lasttime;
 		if (tdiff > timeunit) {
 			frames++;
 			xdiff += tdiff; // always greater 0 because we decided to let tdiff be greater than timeunit
-			if ((xdiff >= 100)&&(xdiff >= timeunit * 20)) {
+			if ((xdiff >= 500)&&(xdiff >= timeunit * 25)) {
 				output.updateFPS(frames * 1000.0 / xdiff); // There are 1000 ticks / second
 				frames = 0;
 				xdiff = 0;
+				ping();
 			}
 			lasttime += tdiff;
 
+			// Multiplayer code
+			doNetworking();
+
 			// Game status code
 			updateGame(tdiff);
 
@@ -148,6 +134,7 @@ void Framework::handleMouseMove(int x, int y, unsigned char buttons)
 	} else if (buttons & SDL_BUTTON(SDL_BUTTON_MIDDLE))
 	{
 		camera.adjustDistance((double)y / 10.0);
+		resetGL();
 	} else {
 		if (!paused) movePaddle((double)x / 50.0, (double)-y / 50.0, SDL_GetTicks());
 		if (camera.getMode() == Camera::FOLLOW_PADDLE)
@@ -159,14 +146,14 @@ void Framework::handleMouseMove(int x, int y, unsigned char buttons)
 
 void Framework::togglePause(bool pause, bool external)
 {
-	Buffer sbuf;
+	PacketType t;
 	if ((((external)&&(pause))||((!external)&&(paused == 0)))&&(state != CONNECTING))
 	{
 		paused = SDL_GetTicks() - lasttime;
 		SDL_ShowCursor(1);
 		SDL_WM_GrabInput(SDL_GRAB_OFF);
 		output.togglePaused(true);
-		sbuf.setType(PAUSE_REQUEST);
+		t = PAUSE_REQUEST;
 	} else {
 		lasttime = SDL_GetTicks() + paused;
 		paused = 0;
@@ -175,10 +162,10 @@ void Framework::togglePause(bool pause, bool external)
 			SDL_WM_GrabInput(SDL_GRAB_ON);
 		}
 		output.togglePaused(false);
-		sbuf.setType(RESUME_REQUEST);
+		t = RESUME_REQUEST;
 	}
 	if (!external)
-		for (int i = 0; i < peer.size(); i++) sendPacket(&peer[i], sbuf.getData(), sbuf.getSize());
+		sendSimplePacket(t);
 }
 
 /* function to load in bitmap as a GL texture */
@@ -190,29 +177,46 @@ GLuint Framework::loadTexture(const std::string& filename)
 	SDL_Surface *image;
 
 	/* trying to load the bitmap */
-	if ((image = SDL_LoadBMP(fullname.c_str())))
+	if ((image = IMG_Load(fullname.c_str())))
 	{
-		/* Create the texture */
-		glGenTextures(1, &texture);
+		/* As the generated SDL surface has the format of the input image,
+		   ie 24 bit or 8 bit, we have to convert it to the format
+		   GL expects from us. In this case, 24 bpp.
+		*/
+		SDL_PixelFormat format;
+		format.palette = NULL;
+		format.BitsPerPixel= 24;
+		format.Rshift= 16;	format.Gshift= 8;	format.Bshift= 0;
+		format.Rmask= 0xff<<16;	format.Gmask= 0xff<<8;	format.Bmask= 0xff;
 
-		/* Load in texture */
-		glBindTexture(GL_TEXTURE_2D, texture);
+		SDL_Surface *teximage = SDL_ConvertSurface(image, &format, SDL_SWSURFACE);
+		if (teximage) {
+			/* Create the texture */
+			glGenTextures(1, &texture);
 
-		/* Nearest filtering looks good on our wall texture */
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+			/* Load in texture */
+			glBindTexture(GL_TEXTURE_2D, texture);
 
-		/* Generate the texture */
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image->w, image->h, 0, GL_BGR,
-				GL_UNSIGNED_BYTE, image->pixels);
+			/* Nearest filtering looks good on our wall texture */
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+
+			/* Generate the texture */
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, teximage->w, teximage->h, 0, GL_BGR,
+					GL_UNSIGNED_BYTE, teximage->pixels);
+
+			SDL_FreeSurface(teximage);
+		} else {
+			std::cerr << "Failed to convert texture format: " << fullname << std::endl;
+			shutdown();
+		}
+
+		SDL_FreeSurface(image);
 	} else {
 		// we have no proper way to deal with missing textures, so we quit
-		std::cerr << "Couldn't load texture: " << fullname << std::endl;
+		std::cerr << "Couldn't load texture: " << fullname << "\n\t" << IMG_GetError() << std::endl;
 		shutdown();
 	}
-	/* Free up any memory we may have used */
-	if (image)
-		SDL_FreeSurface(image);
 
 	return texture;
 }
@@ -230,8 +234,8 @@ void Framework::resetGL()
 	/* Depth buffer setup */
 	glClearDepth(1.0f);
 
-	/* Enable normalizing of normals */
-	glEnable(GL_NORMALIZE);
+	/* Enable normalizing of normals, actually not needed */
+	//glEnable(GL_NORMALIZE);
 
 	/* type of depth test to do */
 	glDepthFunc(GL_LEQUAL);
@@ -259,20 +263,23 @@ void Framework::resetGL()
 	glLightfv(GL_LIGHT3, GL_DIFFUSE, l_diffuse);
 	glLightfv(GL_LIGHT3, GL_SPECULAR, l_specular);
 
+	// translate according actual camera position
+	camera.translate();
+
 	{
-		GLfloat l_pos[] = { -field.getWidth(), -field.getHeight(), -camera.getDistance(), 1.0f };
+		GLfloat l_pos[] = { -field.getWidth(), -field.getHeight(), 0.0f, 1.0f };
 		glLightfv(GL_LIGHT0, GL_POSITION, l_pos);
 	}
 	{
-		GLfloat l_pos[] = { -field.getWidth(), field.getHeight(), -camera.getDistance(), 1.0f };
+		GLfloat l_pos[] = { -field.getWidth(), field.getHeight(), 0.0f, 1.0f };
 		glLightfv(GL_LIGHT1, GL_POSITION, l_pos);
 	}
 	{
-		GLfloat l_pos[] = { field.getWidth(), -field.getHeight(), -camera.getDistance(), 1.0f };
+		GLfloat l_pos[] = { field.getWidth(), -field.getHeight(), 0.0f, 1.0f };
 		glLightfv(GL_LIGHT2, GL_POSITION, l_pos);
 	}
 	{
-		GLfloat l_pos[] = { field.getWidth(), field.getHeight(), -camera.getDistance(), 1.0f };
+		GLfloat l_pos[] = { field.getWidth(), field.getHeight(), 0.0f, 1.0f };
 		glLightfv(GL_LIGHT3, GL_POSITION, l_pos);
 	}
 
@@ -371,12 +378,12 @@ void Framework::drawScene()
 		// we conclude with translucent objects, first in the back:
 
 		for (int i = 0; i < player.size(); i++)
-			if (player[i].getSide() == BACK) player[i].draw();
+			if (player[i]->getSide() == BACK) player[i]->draw();
 
 		// next the ones in the front:
 
 		for (int i = 0; i < player.size(); i++)
-			if (player[i].getSide() == FRONT) player[i].draw();
+			if (player[i]->getSide() == FRONT) player[i]->draw();
 
 		// these are overlays
 		output.drawScore();
@@ -391,26 +398,24 @@ void Framework::drawScene()
 
 Collision* Framework::detectCol(const Vec3f& position, const Vec3f& speed, double radius)
 {
-	Collision* col = new Collision;
+	Collision* col;
 
 	// test against walls
 	col = field.detectCol(position, speed, radius);
 	if (col != NULL) return col;
 
 	// detection for paddles
-	col = player[0].detectCol(position, speed, radius);
-	if (col != NULL) return col;
-
-	col = player[1].detectCol(position, speed, radius);
-	if (col != NULL) return col;
-
-	delete col;
+	for (int i = 0; i < player.size(); i++)
+	{
+		col = player[i]->detectCol(position, speed, radius);
+		if (col != NULL) return col;
+	}
 
 	// wether the ball is still inside?
 	Side side = field.zOutside(position.z);
 	if (side != NONE)
-		// score() detects itself if it's called multiple times
-		score(side);
+		// doScore() detects itself if it's called multiple times
+		doScore(side);
 	return NULL;
 }
 
@@ -445,59 +450,11 @@ void Framework::removeTimer(int index)
 	*/
 }
 
-unsigned int processTimer(unsigned int intervall, void* data)
+void Framework::sendSimplePacket(PacketType t)
 {
-	SDL_Event event;
-	event.type= SDL_USEREVENT;
-	// we have to assure that this is not a net2 used one
-	event.user.code = NET2_EXTERNAL;
-	event.user.data1 = data;
-	SDL_PushEvent(&event);
-	return intervall;
-}
-
-void Framework::startListen()
-{
-	if (NET2_UDPAcceptOn(listenport, 128) == -1)
-	{
-		std::cerr << "Couldn't open UDP port " << listenport << " for listening!" << std::endl;
-		std::cerr << "This is necessary for networking! You can try another port number with -p" << std::endl;
-		shutdown();
-	}
-}
-
-Peer* Framework::getCreatePeer(unsigned int host)
-{
-	for (int i = 0; i < peer.size(); i++)
-		if (peer[i].host == host)
-			return &peer[i];
-
-	peer.push_back(Peer(host));
-	std::cout << "Peer connected from " << peer.back().hostname << std::endl;
-	return &(peer.back());
-}
-
-Peer* Framework::resolveHost(const std::string& hostname)
-{
-	IPaddress ip;
-	if (NET2_ResolveHost(&ip, (char*)hostname.c_str(), sendport) != -1)
-	{
-		peer.push_back(Peer(ip.host));
-		std::cout << "Server resolved at " << peer.back().hostname << std::endl;
-		return &(peer.back());
-	} else return NULL;
-}
-
-void Framework::sendPacket(Peer* receiver, char* data, int size)
-{
-	unsigned short shorty;
-	// we need the port number in network byte order, otherwise it would be the wrong port
-	SDLNet_Write16(sendport, &shorty);
-	IPaddress ip = {
-		receiver->host,
-		shorty
-	};
-	NET2_UDPSend(&ip, data, size);
+	Buffer sbuf(t);
+	/* up till now, all simple packets are state control and therefor need to be sent reliable. */
+	sendPacket(sbuf, true);
 }
 
 void Framework::shutdown()
@@ -506,3 +463,12 @@ void Framework::shutdown()
 	killer.type = SDL_QUIT;
 	SDL_PushEvent(&killer);
 }
+
+unsigned int processTimer(unsigned int intervall, void* data)
+{
+	SDL_Event event;
+	event.type= SDL_USEREVENT;
+	event.user.data1 = data;
+	SDL_PushEvent(&event);
+	return intervall;
+}
