@@ -2,6 +2,7 @@
 #include <iostream>
 #include <sstream>
 #include <getopt.h>
+#include "config.h"
 
 #include "Server.hpp"
 #include "Client.hpp"
@@ -18,6 +19,7 @@
 \n -h \t set y resolution in pixels (default: 768)\
 \n -b \t set individual bitsperpixel (default: 32)\
 \n -f \t operate in fullscreen mode (default: windowed, toggle with 'f' key)\
+\n -v \t show version information and exit\
 \n"
 
 //! function to release used resources and hopefully restore the desktop
@@ -25,11 +27,11 @@
 void Quit(void)
 {
 	/* shutdown net2 */
-	NET2_Quit();
+	//NET2_Quit();
 	// FE_Quit();
 
 	/* shutdown sdlnet which was used by net2 */
-	SDLNet_Quit();
+	//SDLNet_Quit();
 	/* clean up the window */
 	SDL_Quit();
 }
@@ -38,8 +40,9 @@ void Quit(void)
 int main (int argc, char **argv)
 {
 	Configuration conf;
+	std::cout << "Pong2 version " << VERSION << " (network protocol version " << conf.version << ")\n";
 	int c;
-	while ((c = getopt(argc, argv, "c:p:w:h:b:fn:")) != EOF) {
+	while ((c = getopt(argc, argv, "c:p:w:h:b:fn:v")) != EOF) {
 		std::stringstream hlp;
 		switch (c) {
 		case 'c':
@@ -68,6 +71,9 @@ int main (int argc, char **argv)
 		case 'n':
 			conf.playername = optarg;
 			break;
+		case 'v':
+			exit(1);
+			break;
 		default:
 			std::cerr << "Usage: " << argv[0] << " " << USAGE;
 			exit(1);
@@ -103,7 +109,7 @@ int main (int argc, char **argv)
 	SDL_WM_SetCaption("Pong", NULL);
 
 	/* Initialize the networking - net2 is based on sdlnet */
-	if (SDLNet_Init() == -1)
+	/*if (SDLNet_Init() == -1)
 	{
 		std::cerr << "Can't initialize networking (Stage 1): " << SDLNet_GetError() << std::endl;
 		exit(EXIT_FAILURE);
@@ -113,7 +119,7 @@ int main (int argc, char **argv)
 	{
 		std::cerr << "Can't initialize networking (Stage 2): " << NET2_GetError() << std::endl;
 		exit(EXIT_FAILURE);
-	}
+	}*/
 
 	/* create the Server or Client object which controls the game */
 	if (conf.mode == Configuration::SERVER)
